<?php if (!defined('PmWiki')) exit();
/*
******************************************************************************************
*                                                                                        *
*    TrueLatex - Enables interpretation and rendering of real LaTeX markups in PmWiki.   *
*    Copyright (C) 2009  Subhrajit Bhattacharya                                          *
*                                                                                        *
*    This program is free software: you can redistribute it and/or modify                *
*    it under the terms of the GNU General Public License as published by                *
*    the Free Software Foundation, either version 3 of the License, or                   *
*    (at your option) any later version.                                                 *
*                                                                                        *
*    This program is distributed in the hope that it will be useful,                     *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of                      *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                       *
*    GNU General Public License for more details.                                        *
*                                                                                        *
*    You should have received a copy of the GNU General Public License                   *
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.               *
*                                                                                        *
*    Contact: subhrajit@gmail.com, http://fling.seas.upenn.edu/~subhrabh/                *
*                                                                                        *
*    For installation and usage instructions see README.txt                              *
*                                                                                        *
******************************************************************************************
*/

$TrueLatex->TemplateFile = "default.tmpl";  // TODO: Change TEX template file
$TrueLatex->StyleFile = array("geometry.sty"); // TODO: Add style/package files

$TrueLatex->CodeFolder = $FarmD.'/cookbook/TrueLatex/';
$TrueLatex->CacheFolder = $FarmD.'/pub/latexcache/';
$TrueLatex->CacheURL = $PubDirUrl.'/latexcache/';

$TrueLatex->RenderMethod = "pdflatex:convert";
$TrueLatex->OutImageExt = "png";

// =======================================================================

// markup that handels LaTeX commands
Markup(
	'truelatex',
	'fulltext',
	'/\\(:latex(.*?):\\)(.*?)\\(:latexend:\\)/esi',
	"'<:block>'.Keep(TrueLatex_Render(PSS('$2'),PSS('$1')))");

// =======================================================================

function TrueLatex_Render($LatexCode, $OptionString)
{
	global $TrueLatex;
	$LatexCode = htmlspecialchars_decode($LatexCode);
	ExtractOptions($OptionString);
	
	$TrueLatexError = false;
	$TrueLatex->OutMsg = array("");
	
	$ImageFile = $TrueLatex->CacheFolder.md5($LatexCode).'.'.$TrueLatex->OutImageExt;
	$img_path_parts = pathinfo($ImageFile);
	
	if ($TrueLatex->LocalOptions['ClearCache'])
		foreach (glob($TrueLatex->CacheFolder."*.".$TrueLatex->OutImageExt) as $filename)
			if (file_exists($filename))
				unlink($filename);
	
	$attemptedRender = false;
	if (!file_exists($ImageFile) || $TrueLatex->LocalOptions['ForceRender'])
	{
		$attemptedRender = true;
		$TempTEXfile = TrueLatex_PrepareCode($LatexCode);  // Create temp-i.tex
		$tex_path_parts = pathinfo($TempTEXfile);
		
		// Create temp-i.pdf, temp-i.aux, temp-i.log, etc. :
		$TrueLatex->OutMsg[] = "<b>Parsing LaTeX:</b>\n";
		$DocBaseName = TrueLatex_parseTeX($TrueLatex->CacheFolder, $tex_path_parts['filename']);
		if (!file_exists($TrueLatex->CacheFolder.$DocBaseName))
		{
			$TrueLatexError = true;
			$TrueLatex->OutMsg[] = "<b>ERROR in parsing LaTeX!</b>";
		}
		
		// Create xxxxxxxxx...xxxx.png
		$TrueLatex->OutMsg[] = "\n";
		$TrueLatex->OutMsg[] = "<b>Extracting image:</b>\n";
		if (!$TrueLatexError)
			TrueLatex_extractPNG($TrueLatex->CacheFolder, $DocBaseName, $img_path_parts['basename']);
		else
			$TrueLatex->OutMsg[] = "Aborted!";
		
		// Delete temp-i.*
		foreach (glob($tex_path_parts['dirname']."/".$tex_path_parts['filename'].".*") as $filename)
			if (file_exists($filename))
				unlink($filename);
		
	}
	
	if (!file_exists($ImageFile) && !$TrueLatexError && $attemptedRender)
	{
		$TrueLatexError = true;
		$TrueLatex->OutMsg[] = "\n<b>ERROR in extracting image!</b>\n";
	}
	elseif (!$TrueLatexError && $attemptedRender)
		$TrueLatex->OutMsg[] = "Done!";
		
	if (!$attemptedRender)
		$TrueLatex->OutMsg[] = "Rendering was not performed since file existed in cache. You may want to set ForceRender=true";
	
	$ImageURL = $TrueLatex->CacheURL.$img_path_parts['basename'];
	
	$OutHTML = "";
	
	if (!$TrueLatex->LocalOptions['DisplayCode'] || $TrueLatex->LocalOptions['DisplayCodePlus'])
		$OutHTML = $OutHTML."<img class='TrueLatexImage' border=0 src=\"".$ImageURL."\" />";
		
	if ($TrueLatex->LocalOptions['DisplayCode'] || $TrueLatex->LocalOptions['DisplayCodePlus'])
		$OutHTML = $OutHTML."<div class='TrueLatexCode'><pre><code>".htmlspecialchars($LatexCode)."</code></pre></div>";
		
	if ($TrueLatex->LocalOptions['Verbose'])
		$OutHTML = $OutHTML."</br><div class='TrueLatexOutput'><pre><code>\n<b><u>TrueLatex:</u></b><br>".implode($TrueLatex->OutMsg,"\n   ")."</code></pre></div>";
		
	return($OutHTML);
}


function ExtractOptions($OptionString)
{
	global $TrueLatex;
	
	$TrueLatex->LocalOptions['ForceRender'] = false;
	if (preg_match('/ForceRender[ \t]*=[ \t"\']*true[ \t"\']*/i', $OptionString))
		$TrueLatex->LocalOptions['ForceRender'] = true;
		
	$TrueLatex->LocalOptions['ClearCache'] = false;
	if (preg_match('/ClearCache[ \t]*=[ \t"\']*true[ \t"\']*/i', $OptionString))
		$TrueLatex->LocalOptions['ClearCache'] = true;
		
	$TrueLatex->LocalOptions['DisplayCode'] = false;
	if (preg_match('/DisplayCode[ \t]*=[ \t"\']*true[ \t"\']*/i', $OptionString))
		$TrueLatex->LocalOptions['DisplayCode'] = true;
		
	$TrueLatex->LocalOptions['DisplayCodePlus'] = false;
	if (preg_match('/DisplayCode[ \t]*\+[ \t]*=[ \t"\']*true[ \t"\']*/i', $OptionString))
		$TrueLatex->LocalOptions['DisplayCodePlus'] = true;
		
	$TrueLatex->LocalOptions['Verbose'] = false;
	if (preg_match('/Verbose[ \t]*=[ \t"\']*true[ \t"\']*/i', $OptionString))
		$TrueLatex->LocalOptions['Verbose'] = true;
		
}


function TrueLatex_PrepareCode($LatexBody, $LatexHeader="")
{
	global $TrueLatex;
	
	foreach ($TrueLatex->StyleFile as $StyleFile)
	{
		$OriginalStyleFile = $TrueLatex->CodeFolder.$StyleFile;
		$CopiedStyleFile = $TrueLatex->CacheFolder.$StyleFile;
		if (!file_exists($CopiedStyleFile) || 
			  !(filesize($OriginalStyleFile)==filesize($CopiedStyleFile)) || 
			    !(filectime($OriginalStyleFile)==filectime($CopiedStyleFile)))
			copy($OriginalStyleFile, $CopiedStyleFile);
	}
		
	$TempFileTry = 0;
	do {
		$TempTEXfile = $TrueLatex->CacheFolder."temp-".$TempFileTry.".tex";
		$TempFileTry++;
	} while(file_exists($TempTEXfile) && $TempFileTry<100);
	
	$TEXfileContent = file_get_contents($TrueLatex->CodeFolder.$TrueLatex->TemplateFile);
	$TEXfileContent = str_replace("<$-Body-$>", $LatexBody, $TEXfileContent);
	$TEXfileContent = str_replace("<$-Header-$>", $LatexHeader, $TEXfileContent);
	
	file_put_contents($TempTEXfile, $TEXfileContent);
	return $TempTEXfile;
}

// =======================================================================

function TrueLatex_parseTeX($dirname, $filename)
{
	global $TrueLatex;
	$thisMethod = split(':', $TrueLatex->RenderMethod);
	
	switch ($thisMethod[0])
	{
		case "pdflatex":
			exec(	"cd ".$dirname ."; ".
					"pdflatex ".$filename,
											$TrueLatex->OutMsg);
			return $filename.'.pdf';
	}
}


function TrueLatex_extractPNG($dirname, $name_in, $name_out)
{
	global $TrueLatex;
	$thisMethod = split(':', $TrueLatex->RenderMethod);
	
	switch ($thisMethod[1])
	{
		case "convert":
			exec(	"cd ".$dirname ."; ".
					"convert ".$name_in." -trim ".$name_out,
															$TrueLatex->OutMsg);
			break;
	}
}

?>
